package com.urbancode.anthill3.plugin.subversion

import com.urbancode.anthill3.*

import java.text.*;
import java.util.Date;

public class SCMCheckout extends SCMStep {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    //**************************************************************************
    // INSTANCE
    //**************************************************************************

    String fromTag
    String depth
    boolean cleanWorkspace
    boolean ignoreExternals
    boolean doExport

    public void execute() {
        PerfLogger perf = new PerfLogger(System.out, 'SCMCheckout#execute');
        try {
            initWorkspace();

            // CLEANUP
            if (cleanWorkspace && directory.exists()) {
                System.out.println("Deleting $directory");
                delete(directory);
            }
            directory.mkdirs()
            perf.trace("Cleaned/created WC directory");

            if (!directory.exists()) {
                throw new Exception("Could not create working directory ${directory}")
            }
            else if (directory.isFile()) {
                throw new Exception("Specified working directory is a file!! ${directory}")
            }

            CommandHelper cmdHelper = getCommandHelperBase();

            // populate last-changed revs etc for back-compatibility
            initSvnInfo();
            perf.trace("got svn info (as needed)");
            if (!revision) {
                revision = currentRevision;
            }

            //
            // CHECKOUT/EXPORT
            //
            def command = getBaseCmd();
            if (doExport) {
                command << 'export' << '--force' // need force because target directory does/may exist
            }
            else {
                command << 'checkout'
            }
            if (depth && depth != "infinity") {
                command << '--depth' << depth
            }
            if (ignoreExternals) {
                command << '--ignore-externals'
            }
            assert revision : 'Could not determine revision';
            command << '--revision' << revision
            command << targetUrl;
            command << '.'
            cmdHelper.runCommand('Subversion Populate', command);
            perf.trace("completed svn populate");
        }
        finally {
            perf.trace("complete");
        }
    }
}
