package com.urbancode.anthill3.plugin.subversion

public class SCMCleanup extends SCMStep {

    //**************************************************************************
    // CLASS
    //**************************************************************************

    //**************************************************************************
    // INSTANCE
    //**************************************************************************

    boolean allFiles = true;
    boolean svnCleanup = true;
    boolean remUnversionedFiles = true;
    boolean revertFiles = true;

    public void execute() {
        initWorkspace();

        // All Files
        if (!directory.listFiles()) {
            println "$directory is empty, nothing to cleanup"
        }
        else if (allFiles) {
            println "Cleanup all files is selected (other operations not applicable)."
            println "Deleting directory contents for $directory"
            directory.listFiles().each{delete(it)};
            println "Done deleting directory contents for $directory"
        }
        else if (!new File(directory, '.svn').exists()) {
            println "Directory $directory.path is not an svn workspace, deleting all files";
            println "Deleting directory contents for $directory"
            directory.listFiles().each{delete(it)};
            println "Done deleting directory contents for $directory"
        }
        else {
            def cmdHelper = getCommandHelperBase();

            // svn cleanup
            if (svnCleanup) {
                def stCmd = getBaseCmd();
                stCmd << 'cleanup' << directory;
                cmdHelper.runCommand('Performing "svn cleanup" operation on $dir', stCmd)
            }
            // Remove unversioned files
            if (remUnversionedFiles) {
                def stOut = null;
                def stCmd = getBaseCmd();
                stCmd << 'status';
                cmdHelper.runCommand("Removing unversioned files from $directory", stCmd) { Process proc ->

                    proc.out.close() // close stdin
                    def errThread = proc.consumeProcessErrorStream(System.err);
                    try {
                        stOut = proc.text;

                        // trim trailing whitespace in output, otherwise we may get extra consecuative line endings
                        println stOut.replaceFirst(~/\s+$/, '');
                    }
                    finally {
                        if (errThread != null) {
                            errThread.join();
                        }
                    }
                }

                // find lines with unversioned files
                stOut.eachLine{ line ->
                    // line starts with ?
                    if (line =~ /^\?/) {
                        String fileName = line.replaceFirst(~/^\?       /, "");
                        File file = new File(directory, fileName);
                        delete(file);
                    }
                }

                println "Done deleting unversioned files"
            }

            // Revert files
            if (revertFiles) {
                def rvt = getBaseCmd();
                rvt << 'revert' << '--recursive' << directory;
                cmdHelper.runCommand("Reverting all files in $directory", rvt);
                println "Done reverting files"
            }
        }
    }
}
