import java.text.SimpleDateFormat
import java.util.regex.Pattern
import java.util.regex.Matcher
import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.subversion.*
import com.urbancode.anthill3.plugin.subversion.changelog.*;

final def out = System.out
File workDir = new File('.').canonicalFile
AHPTool ahptool = new AHPTool()
DateParser dateParser = new DateParser()

//------------------------------------------------------------------------------
// PARSE PROPERTIES
//------------------------------------------------------------------------------

final def props = ahptool.getStepProperties()// + ahptool.getJobProperties()
final def blProps = ahptool.getBuildLifeProperties();

// Repo
String repositoryUrlBase = props['source/repo/repositoryUrl']
String commandPath       = props['source/repo/commandPath']
String repoUserName      = props['source/repo/username']
String repoPassword      = props['source/repo/password']
boolean trustServerCert  = Boolean.valueOf(props['source/repo/trustServerCert'])
boolean isSvnRepo        = Boolean.valueOf(props['source/repo/isSvnRepo'])

// Source
String srcName           = props['source'];
File dir                 = new File(workDir, props['source/dirOffset']?: '.').absoluteFile // checkout directory
String checkoutPath      = props['source/projectTrunkPath']
String branchesDirOffset = props['source/projectBranchesPath']
String tagsPath          = props['source/projectTagsPath']
String revision          = props['source/revision'];
String noCache           = Boolean.valueOf(props['source/cache']);
boolean ignoreExternals  = Boolean.valueOf(props['source/ignoreExternals'])
boolean cleanWorkspace   = Boolean.valueOf(props['source/cleanWorkspace'])
//boolean export           = Boolean.valueOf(props['source/export'])
String sourceUserFilters = props['source/excludeUsers'] ?: ''
String sourceFileFilters = props['source/fileFilters'] ?: ''

// STEP
String stepNoCache = props['cache']
String fromTag       = props['fromTag']
String startRevision = props['startRevision']
String endRevision   = props['endRevision']
Date startDate     = dateParser.parseDate(props['startDate']) // TODO fallback to property:workspace.date?
Date endDate       = dateParser.parseDate(props['endDate']) // TODO fallback to property:workspace.date?
String startBuildlifeId = props['startBuildlifeId'];
String stepUserFilters = props['excludeUsers']
String stepFileFilters     = props['fileFilters']

// hybrid values
String userFilters = sourceUserFilters + '\n' + stepUserFilters;
String fileFilters = sourceFileFilters + '\n' + stepFileFilters;
boolean doNoCache = stepNoCache != null ? Boolean.valueOf(stepNoCache) : noCache;

String blRevision = blProps['workspace.revision.'+srcName];
String startBlRevision = null;
if (startBuildlifeId) {
    def startBlProps =ahptool.getBuildLifeProperties(startBuildlifeId)
    startBlRevision = startBlProps['workspace.revision.'+srcName]
}


//
// Execute
//

SCMChangelog step = new SCMChangelog();
step.directory  = new File('.'); // ignore local directory
step.scmCommand = commandPath;
step.svnBaseUrl = repositoryUrlBase;
step.svnUser = repoUserName;
step.svnPass = repoPassword;
step.noAuthCache = doNoCache;
step.trustServerCert = trustServerCert;
step.checkoutPath = checkoutPath;
step.tagsPath = tagsPath

step.startRevision = startBlRevision ?: startRevision
step.endRevision = blRevision ?: endRevision
step.startDate = startDate
//step.endDate = endDate // Use workspace date as end point
step.changeSetFilter.setUserExcludes(userFilters);
step.changeSetFilter.setFileFilters(fileFilters);

if (revision) {
    String msg = "Skipping changelog for revision based checkout for %s:r%s";
    println(String.format(msg, step.targetUrl, revision));
    return;
}

def xml = step.execute()
println("") // spacing
if (xml) {
    ahptool.addChangeSets(xml);
    println "";
}
else {
    println "No changes detected"
}
