#!/usr/bin/env groovy

import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.subversion.*

final def out = System.out
File workDir = new File('.').canonicalFile
AHPTool ahpTool = new AHPTool();
DateParser dateParser = new DateParser();

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------

def perfLogger = new PerfLogger(System.out, "step");
try {

    final def props = ahpTool.getStepProperties();
    //final def jobProps = ahpTool.getJobProperties();
    final def blProps = ahpTool.getBuildLifeProperties();

    perfLogger.trace("completed getting properties");

    // Repo
    String repositoryUrlBase = props['source/repo/repositoryUrl']
    String commandPath       = props['source/repo/commandPath']
    String repoUserName      = props['source/repo/username']
    String repoPassword      = props['source/repo/password']
    boolean trustServerCert  = Boolean.valueOf(props['source/repo/trustServerCert'])
    boolean isSvnRepo        = Boolean.valueOf(props['source/repo/isSvnRepo'])

    // Source
    String srcName           = props['source'];
    File dir                 = new File(workDir, props['source/dirOffset']?: '.').absoluteFile // checkout directory
    String checkoutPath      = props['source/projectTrunkPath']
    String branchesDirOffset = props['source/projectBranchesPath']
    String tagsPath          = props['source/projectTagsPath']
    String revision          = props['source/revision'];
    String noCache           = Boolean.valueOf(props['source/cache']);
    boolean ignoreExternals  = Boolean.valueOf(props['source/ignoreExternals'])
    boolean cleanWorkspace   = Boolean.valueOf(props['source/cleanWorkspace'])
    boolean export           = Boolean.valueOf(props['source/export'])
    String sourceUserFilters = props['source/excludeUsers'] ?: ''
    String sourceFileFilters = props['source/fileFilters'] ?: ''

    // STEP
    Date date = dateParser.parseDate(props['date']) // this is from actualWorkspaceDate

    String stepNoCache = props['cache']
    String fromTag = props['fromTag']
    String stepClean = props['cleanWorkspace'];
    String stepExport = props['export'];
    String stepRev = props['revision']
    String depth = props['depth'] ?: 'infinity'

    // BUILDLIFE
    final String blRevision = blProps["workspace.revision.$srcName"];

    // hybrid values
    boolean doNoCache = stepNoCache != null ? Boolean.valueOf(stepNoCache) : noCache;
    boolean doClean = stepClean != null ? Boolean.valueOf(stepClean) : cleanWorkspace;
    boolean doExport = stepExport != null ? Boolean.valueOf(stepExport) : export;

    //
    // Perform step
    //
    dir.mkdirs();

    SCMCheckout step = new SCMCheckout();
    step.directory  = dir;
    step.scmCommand = commandPath;
    step.revision   = stepRev ?: blRevision ?: revision;
    step.date       = date;

    step.svnBaseUrl = repositoryUrlBase;
    step.svnUser = repoUserName;
    step.svnPass = repoPassword;
    step.noAuthCache = doNoCache;
    step.trustServerCert = trustServerCert;
    step.checkoutPath = checkoutPath;
    step.tagsPath = tagsPath

    step.fromTag = fromTag
    step.depth = depth
    step.cleanWorkspace = doClean
    step.ignoreExternals = ignoreExternals
    step.doExport = doExport

    perfLogger.trace("starting SCMCheckout");
    step.execute();
    println("") // spacing
    perfLogger.trace("completed SCMCheckout");

    //
    // Update BL Properties on server
    //

    if (stepRev || fromTag) {
        println "A step setting overriding revision or tag was used. Skipping update of build-life properties"
        return
    }

    perfLogger.trace("getting current buildlife properties");
    def currentBuildLifeProps = ahpTool.getBuildLifeProperties();
    perfLogger.trace("complete getting current buildlife properties");
    def updateBlProps = [
        // legacy properties for user reference only
        'svn-revision' : step.currentRevision,
        'svn-changed-revision' : step.lastChangedRevision
    ]
    if (step.currentRevision) {
        updateBlProps['workspace.revision.'+srcName] = step.currentRevision;
    }

    updateBlProps.values().remove(null);
    updateBlProps.values().remove('');
    updateBlProps.entrySet().removeAll(currentBuildLifeProps.entrySet())
    if (updateBlProps) {
        println "Updating buildlife properties for revision/date of source"
        ahpTool.setBuildLifeProperties(updateBlProps)
        perfLogger.trace("completed update of buildlife properties");
    }
}
finally {
    perfLogger.trace("step complete");
}