#!/usr/bin/env groovy

import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.subversion.*

final def workDir = new File('.').canonicalFile
final def out = System.out
final def env = System.getenv()

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
AHPTool ahpTool = new AHPTool()

final def props = ahpTool.getStepProperties();
final def blProps = ahpTool.getBuildLifeProperties();

// Repo
String repositoryUrlBase = props['source/repo/repositoryUrl']
String commandPath       = props['source/repo/commandPath']
String repoUserName      = props['source/repo/username']
String repoPassword      = props['source/repo/password']
boolean trustServerCert  = Boolean.valueOf(props['source/repo/trustServerCert'])
boolean isSvnRepo        = Boolean.valueOf(props['source/repo/isSvnRepo'])

// Source
String srcName           = props['source'];
File dir                 = new File(workDir, props['source/dirOffset']?: '.').absoluteFile // checkout directory
String branchesDirOffset = props['source/projectBranchesPath']
String tagsPath          = props['source/projectTagsPath']
String noCache           = Boolean.valueOf(props['source/cache']);

// STEP
String stepNoCache = props['cache']
String createBranch = stepProps['createBranch']
String createTag    = stepProps['createTag']
String message      = stepProps['commitMessage']

// hybrid values
boolean doNoCache = stepNoCache != null ? Boolean.valueOf(stepNoCache) : noCache;


if (createBranch && createTag) {
    throw new Exception('Both a branch and tag can not be specified at the same time')
}

if (!createBranch && !createTag) {
    throw new Exception('Must specify either a branch or a tag')
}

//------------------------------------------------------------------------------
// EXECUTE
//------------------------------------------------------------------------------


SCMTag step = new SCMTag();
step.directory  = dir;
step.scmCommand = commandPath;

step.svnBaseUrl = repositoryUrlBase;
step.svnUser = repoUserName;
step.svnPass = repoPassword;
step.noAuthCache = doNoCache;
step.trustServerCert = trustServerCert;
step.tagsPath = tagsPath
step.branchesPath = branchesDirOffset;

step.execute();
println("")