#!/usr/bin/env groovy

import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.subversion.*
import java.text.*

final def dateParser = new DateParser();
final def ahpTool = new AHPTool();

//------------------------------------------------------------------------------
// PARSE PROPERTIES
//------------------------------------------------------------------------------

File workDir = new File('.').canonicalFile
def out = System.out
def env = System.getenv()
def props = ahpTool.getStepProperties()

//Repo
String repositoryUrlBase = props['source/repo/repositoryUrl']
String commandPath       = props['source/repo/commandPath']
String repoUserName      = props['source/repo/username']
String repoPassword      = props['source/repo/password']
boolean trustServerCert  = Boolean.valueOf(props['source/repo/trustServerCert'])
boolean isSvnRepo        = Boolean.valueOf(props['source/repo/isSvnRepo'])

// Source
String srcName           = props['source'];
File dir                 = new File(workDir, props['source/dirOffset']?: '.').absoluteFile // checkout directory
String checkoutPath      = props['source/projectTrunkPath']
String branchesDirOffset = props['source/projectBranchesPath']
String tagsPath          = props['source/projectTagsPath']
String revision          = props['source/revision'];
String noCache           = Boolean.valueOf(props['source/cache']);
boolean ignoreExternals  = Boolean.valueOf(props['source/ignoreExternals'])
String sourceUserFilters = props['source/excludeUsers'] ?: ''
String sourceFileFilters = props['source/fileFilters'] ?: ''

// STEP
String stepNoCache = props['cache']
String fromTag       = props['fromTag']
String startRevision = props['startRevision']
String endRevision   = props['endRevision']
Date startDate     = dateParser.parseDate(props['startDate'])
Date endDate       = dateParser.parseDate(props['endDate'])
String stepUserFilters = props['excludeUsers']
String stepFileFilters     = props['fileFilters']

String userFilters = sourceUserFilters + '\n' + stepUserFilters;
String fileFilters = sourceFileFilters + '\n' + stepFileFilters;

//hybrid values
boolean doNoCache = stepNoCache != null ? Boolean.valueOf(stepNoCache) : noCache;

//------------------------------------------------------------------------------
// PREPARE COMMAND LINE
//------------------------------------------------------------------------------
//  svn log --revision '{2008-12-22 08:09:39 -0500}:{2008-12-15 09:47:55 -0500}' --verbose --xml path
//     note: can not use the "--limit X" flag because the latest change could be from an ignored author or contain only excluded files
//


SCMQuietPeriod step = new SCMQuietPeriod();
step.directory  = new File('.'); // UNUSED
step.scmCommand = commandPath;
step.revision   = revision;
step.svnBaseUrl = repositoryUrlBase;
step.svnUser = repoUserName;
step.svnPass = repoPassword;
step.noAuthCache = doNoCache;
step.trustServerCert = trustServerCert;
step.checkoutPath = checkoutPath;
step.tagsPath = tagsPath

step.startRevision = startRevision
step.endRevision = endRevision
step.startDate = startDate
step.endDate = endDate

step.changeSetFilter.setUserExcludes(userFilters);
step.changeSetFilter.setFileFilters(fileFilters);

if (revision) {
    String msg = "Skipping quiet period adjustment for revision based checkout for %s:r%s";
    println(String.format(msg, step.targetUrl, revision));
    return;
}

Date latestRevisionDate = step.execute()

//------------------------------------------------------------------------------
// UPLOAD LATEST CHANGE DATE
//------------------------------------------------------------------------------
if (latestRevisionDate == null) {
    println('No changes detected')
}
else {
    ahpTool.setQuietPeriodLatestTime(latestRevisionDate.time)
    println('')
}
