import com.urbancode.anthill3.*
import com.urbancode.anthill3.plugin.subversion.*

final def workDir = new File('.').canonicalFile
final def out = System.out
final def env = System.getenv()

//------------------------------------------------------------------------------
// GET ALL INPUT PARAMETERS
//------------------------------------------------------------------------------
AHPTool ahpTool = new AHPTool()
DateParser dateParser = new DateParser();

final def props = ahpTool.getStepProperties();
final def blProps = ahpTool.getBuildLifeProperties();

// Repo
String repositoryUrlBase = props['source/repo/repositoryUrl']
String commandPath       = props['source/repo/commandPath']
String repoUserName      = props['source/repo/username']
String repoPassword      = props['source/repo/password']
boolean trustServerCert  = Boolean.valueOf(props['source/repo/trustServerCert'])

// Source
String srcName           = props['source'];
String checkoutPath      = props['source/projectTrunkPath']
String branchesDirOffset = props['source/projectBranchesPath']
String tagsPath          = props['source/projectTagsPath']
String revision          = props['source/revision'];
String noCache           = Boolean.valueOf(props['source/cache']);

// STEP
String stepNoCache = props['cache']
String stepRev = props['revision']
String createBranch = stepProps['createBranch']
String createTag    = stepProps['createTag']
String message      = stepProps['commitMessage']
String fromBranch   = stepProps['fromBranch']
String fromTag      = stepProps['fromTag']

// BUILDLIFE
final String blRevision = blProps["workspace.revision.$srcName"];

// hybrid values
boolean doNoCache = stepNoCache != null ? Boolean.valueOf(stepNoCache) : noCache;
Date date           = dateParser.parseDate(stepProps['date'] ?: stepProps['workspace.date'])

if (createBranch && createTag) {
    throw new Exception('Both a branch and tag can not be specified at the same time')
}

if (!createBranch && !createTag) {
    throw new Exception('Must specify either a branch or a tag')
}

//------------------------------------------------------------------------------
// EXECUTE
//------------------------------------------------------------------------------

SCMRemoteTag step = new SCMRemoteTag();
step.directory = new File('.'); // UNUSED
step.scmCommand = commandPath;
step.revision   = stepRev ?: blRevision ?: revision;
step.date       = date;

step.svnBaseUrl = repositoryUrlBase;
step.svnUser = repoUserName;
step.svnPass = repoPassword;
step.noAuthCache = doNoCache;
step.trustServerCert = trustServerCert;
step.checkoutPath = checkoutPath;
step.tagsPath = tagsPath
step.fromTag = fromTag
step.fromBranch = fromBranch;
step.branchesPath = branchesDirOffset;

step.execute();
println("")